##############################################################################
#
# Auswertungstool fuer Atlantis Computer-Reports
# Gemeinsame Routinen
# $ProjectVersion: 1.0.8 $   
# $ProjectDate: Tue, 03 Oct 2006 23:26:16 +0200 $
# Copyright: Norbert Donath   <Norbert.Donath@sigos.de>
#
#############################################################################


##############################################################################
# Name:            ReadConfigFile
# Parameters:      NONE
# Return value:    NONE
# Description:     Reads the configuration file.
#############################################################################

proc ReadConfigFile {} {
  global libDir picArray maxLineLength

  source .auswertrc
}


##############################################################################
# Name:            FileOpenOrExit
# Parameters:      fileName: string
#                  mode: string
# Return value:    retVal: file descriptor
# Description:     Opens the file $fileName with mode $mode and returns the
#                  file descriptor. If the open fails, exit is called.
#############################################################################

proc FileOpenOrExit {fileName mode} {
  if {[catch {set retVal [open $fileName $mode]}]} {
    puts "Datei '$fileName' konnte nicht mit Mode '$mode' geffnet werden"
    exit 1
  }
  return $retVal
}


##############################################################################
# Name:            ReadLineOrExit
# Parameters:      file: file descriptor
# Return value:    retVal: string
# Description:     Reads a line from fileName or exits if gets failes.
#############################################################################

proc ReadLineOrExit {file} {
  if {[gets $file readStr] == -1} {
    exit 1
  }
  return $readStr
}


##############################################################################
# Name:            GetRegionTitle
# Parameters:      region: list
# Return value:    string
# Description:     Generates the title string for a region of the type
#                  '$regiontype von $regionname' or 'Ozean ($x,$y)' and
#                  returns it.
#############################################################################

proc GetRegionTitle {region} {
  set type [GetValueFromList terrain $region]
  if {$type == "Ozean"} {
    set x [GetValueFromList x $region]
    set y [GetValueFromList y $region]
    return "Ozean ($x,$y)"
  } elseif {$type == ""} {
    return "Unbekannte Region"
  } else {
    set name [GetValueFromList name $region]
    return "$type von $name"
  }
}


##############################################################################
# Name:            ComputeJobs
# Parameters:      region: list
# Return value:    ---: int
# Description:     Computes the maximum number of jobs within the region.
#############################################################################

proc ComputeJobs {region} {
  set terrain [GetValueFromList terrain $region]
  set trees [GetValueFromList baeume $region]
  switch $terrain {
    Ebene -
    Wald {set maxJobs 10000}
    Hochland {set maxJobs 4000}
    Sumpf {set maxJobs 2000}
    Berge {set maxJobs 1000}
    Wueste {set maxJobs 500}
    Gletscher {set maxJobs 100}
    Ozean {set maxJobs 0}
  }
  set jobs [expr {$maxJobs - 10 * $trees}]
  if {$jobs > 0} {
    return $jobs
  } else {
    return 0
  }
}


##############################################################################
# Name:            GetCastleType
# Parameters:      size: int
# Return value:    ---: string
# Description:     Returns the type of castle
#############################################################################

proc GetCastleType {size} {
  if {$size >= 1250} {
    return "Zitadelle"
  }
  if {$size >= 250} {
    return "Festung"
  }
  if {$size >= 50} {
    return "Schlo"
  }
  if {$size >= 10} {
    return "Turm"
  }
  if {$size >= 2} {
    return "Befestigung"
  }
  if {$size == 1} {
    return "Baustelle"
  }
  return "Hundehtte ;-)"
}


##############################################################################
# Name:            SortUnits
# Parameters:      x: int
#                  y: int
#                  uList: list
# Return value:    NONE
# Description:     Sorts the units in uList into castles and ships
#############################################################################

proc SortUnits {x y uList} {
  global unitList castleArray shipArray regionArray ownNumber alliedList

  set ownList {}
  set aList {}
  set otherList {}
  foreach unitNr $uList {
    set unit [GetValueFromList $unitNr $unitList]
    set castleNr [GetValueFromList burgnr $unit]
    if {$castleNr != 0} {
      lappend castleArray($castleNr) $unitNr
      continue
    }
    set shipNr [GetValueFromList schiffnr $unit]
    if {$shipNr != 0} {
      lappend shipArray($shipNr) $unitNr
      continue
    }
    set party [GetValueFromList partei $unit]
    if {$party == $ownNumber} {
      lappend ownList $unitNr
    } elseif {[lsearch $alliedList $party] != -1} {
      lappend aList $unitNr
    } else {
      lappend otherList $unitNr
    }
  }
  set regionArray($x,$y) {}
  foreach tmp $ownList {
    lappend regionArray($x,$y) $tmp
  }
  foreach tmp $aList {
    lappend regionArray($x,$y) $tmp
  }
  foreach tmp $otherList {
    lappend regionArray($x,$y) $tmp
  }
}


##############################################################################
# Name:            PrintShipLoad
# Parameters:      ship: list
# Return value:    string
# Description:     Computes the load of a ship and returns a string for the
#                  HTML file.
#############################################################################

proc PrintShipLoad {ship} {
  global shipArray capacity

  set load 0
  set number [GetValueFromList nummer $ship]
  set type [GetValueFromList typ $ship]
  if {[info exists shipArray($number)]} {
    foreach unitNumber $shipArray($number) {
      incr load [GetUnitWeight total $unitNumber]
    }
  }
  return "Kapazitt: [expr {$capacity($type) - $load}]/$capacity($type) frei"
}


##############################################################################
# Name:            GetUnitWeight
# Parameters:      mode: ("total" | "ride" | "foot")
#                  unitNumber: int
# Return value:    int
# Description:     Returns the weight of a unit. Mode total means weight of
#                  the persons, goods, horses and carts. Mode ride means the
#                  weight of persons and goods excluding horses and carts.
#                  Mode foot means only the weight of the goods excluding
#                  horses and carts.
#############################################################################

proc GetUnitWeight {mode unitNumber} {
  global unitList goodWeight goodList

  set weight 0
  set unit [GetValueFromList $unitNumber $unitList]
  set type [GetValueFromList typ $unit]
  set amount [GetValueFromList anzahl $unit]
  if {($mode == "total" || $mode == "ride") && [info exists goodWeight($type)]} {
    incr weight [expr {$amount * $goodWeight($type)}]
  }
  foreach good $goodList {
    if {$mode != "total" && ($good == "wagen" || $good == "pferd")} {
      continue
    }
    incr weight [expr {[GetValueFromList $good $unit] * $goodWeight($good)}]
  }
  return $weight
}


##############################################################################
# Name:            GetAmounts
# Parameters:      uList: list
# Return value:    NONE
# Description:     Gets the total number of goods of own units in the region.
##############################################################################

proc GetAmounts {uList} {
  global goodList amounts ownNumber unitList

  foreach good $goodList {
    set amounts($good) 0
  }
  set amounts(silber) 0
  foreach unitNr $uList {
    set unit [GetValueFromList $unitNr $unitList]
    set owner [GetValueFromList partei $unit]
    if {$owner == $ownNumber} {
      foreach good $goodList {
	incr amounts($good) [GetValueFromList $good $unit]
      }
      incr amounts(silber) [GetValueFromList silber $unit]
    }
  }
}

